@@ -89,7 +89,7 @@ GS_EXPORT const NSTimeInterval NSTimeIntervalSince1970;
 - (NSString*) descriptionWithCalendarFormat: (NSString*)format
 				   timeZone: (NSTimeZone*)aTimeZone
 				     locale: (NSDictionary*)l;
-- (NSString*) descriptionWithLocale: (NSDictionary*)locale;
+- (NSString*) descriptionWithLocale: (id)locale;
 
 // Adding and getting intervals
 
@@ -98,7 +98,6 @@ GS_EXPORT const NSTimeInterval NSTimeIntervalSince1970;
 - (NSTimeInterval) timeIntervalSinceDate: (NSDate*)otherDate;
 - (NSTimeInterval) timeIntervalSinceNow;
 - (NSTimeInterval) timeIntervalSinceReferenceDate;
-- (NSTimeInterval) timeIntervalSinceReferenceDate;
 
 // Comparing dates
 
@@ -107,6 +106,14 @@ GS_EXPORT const NSTimeInterval NSTimeIntervalSince1970;
 - (BOOL) isEqualToDate: (NSDate*)other;
 - (NSDate*) laterDate: (NSDate*)otherDate;
 
+#if OS_API_VERSION(100600,GS_API_LATEST)
+/**
+ * Returns an autoreleased NSDate instance whose value is offset from
+ * that of the receiver by the specified interval.
+ */
+- (id) dateByAddingTimeInterval: (NSTimeInterval)ti;
+#endif
+
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 + (id) dateWithNaturalLanguageString: (NSString*)string;
 + (id) dateWithNaturalLanguageString: (NSString*)string
