@@ -3,29 +3,29 @@
 
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
    Date: 1995
-   
+
    This file is part of the GNUstep Base Library.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
-  */ 
+  */
 
 #ifndef _NSDictionary_h_GNUSTEP_BASE_INCLUDE
 #define _NSDictionary_h_GNUSTEP_BASE_INCLUDE
 #import	<GNUstepBase/GSVersionMacros.h>
-
+#import <GNUstepBase/GSBlocks.h>
 #import	<Foundation/NSObject.h>
 #import	<Foundation/NSEnumerator.h>
 
@@ -33,7 +33,7 @@
 extern "C" {
 #endif
 
-@class NSArray, NSString, NSURL;
+@class NSArray, NSSet, NSString, NSURL;
 
 @interface NSDictionary : NSObject <NSCoding, NSCopying, NSMutableCopying, NSFastEnumeration>
 + (id) dictionary;
@@ -44,8 +44,8 @@ extern "C" {
 + (id) dictionaryWithDictionary: (NSDictionary*)otherDictionary;
 + (id) dictionaryWithObject: (id)object forKey: (id)key;
 + (id) dictionaryWithObjects: (NSArray*)objects forKeys: (NSArray*)keys;
-+ (id) dictionaryWithObjects: (id*)objects
-		     forKeys: (id*)keys
++ (id) dictionaryWithObjects: (const id[])objects
+		     forKeys: (const id[])keys
 		       count: (NSUInteger)count;
 + (id) dictionaryWithObjectsAndKeys: (id)firstObject, ...;
 
@@ -59,31 +59,59 @@ extern "C" {
 - (NSString*) descriptionWithLocale: (id)locale
 			     indent: (NSUInteger)level;
 
+#if OS_API_VERSION(100600, GS_API_LATEST)
+DEFINE_BLOCK_TYPE(GSKeysAndObjectsEnumeratorBlock, void, id, id, BOOL*);
+- (void) enumerateKeysAndObjectsUsingBlock:
+  (GSKeysAndObjectsEnumeratorBlock)aBlock;
+- (void) enumerateKeysAndObjectsWithOptions: (NSEnumerationOptions)opts
+  usingBlock: (GSKeysAndObjectsEnumeratorBlock)aBlock;
+#endif
+
+- (void) getObjects: (__unsafe_unretained id[])objects
+            andKeys: (__unsafe_unretained id[])keys;
 - (id) init;
 - (id) initWithContentsOfFile: (NSString*)path;
+
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (id) initWithContentsOfURL: (NSURL*)aURL;
 #endif
+
 - (id) initWithDictionary: (NSDictionary*)otherDictionary;
 - (id) initWithDictionary: (NSDictionary*)other copyItems: (BOOL)shouldCopy;
 - (id) initWithObjects: (NSArray*)objects forKeys: (NSArray*)keys;
 - (id) initWithObjectsAndKeys: (id)firstObject, ...;
-- (id) initWithObjects: (id*)objects
-	       forKeys: (id*)keys
+- (id) initWithObjects: (const id[])objects
+	       forKeys: (const id[])keys
 		 count: (NSUInteger)count;			// Primitive
 - (BOOL) isEqualToDictionary: (NSDictionary*)other;
 
 - (NSEnumerator*) keyEnumerator;				// Primitive
+
+#if OS_API_VERSION(100600, GS_API_LATEST)
+DEFINE_BLOCK_TYPE(GSKeysAndObjectsPredicateBlock, BOOL, id, id, BOOL*);
+- (NSSet*) keysOfEntriesPassingTest: (GSKeysAndObjectsPredicateBlock)aPredicate;
+- (NSSet*) keysOfEntriesWithOptions: (NSEnumerationOptions)opts
+                        passingTest: (GSKeysAndObjectsPredicateBlock)aPredicate;
+#endif
+
 - (NSArray*) keysSortedByValueUsingSelector: (SEL)comp;
 - (NSEnumerator*) objectEnumerator;				// Primitive
 - (id) objectForKey: (id)aKey;					// Primitive
 - (NSArray*) objectsForKeys: (NSArray*)keys notFoundMarker: (id)marker;
 
-- (BOOL) writeToFile: (NSString*)path atomically: (BOOL)useAuxiliaryFile;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (id) valueForKey: (NSString*)key;
+#endif
+
+- (BOOL) writeToFile: (NSString*)path atomically: (BOOL)useAuxiliaryFile;
+
+#if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 - (BOOL) writeToURL: (NSURL*)url atomically: (BOOL)useAuxiliaryFile;
 #endif
+/**
+ * Method called by array subscripting.
+ */
+- (id) objectForKeyedSubscript: (id)aKey;
 @end
 
 @interface NSMutableDictionary: NSDictionary
@@ -98,10 +126,15 @@ extern "C" {
 - (void) setObject: (id)anObject forKey: (id)aKey;		// Primitive
 - (void) setDictionary: (NSDictionary*)otherDictionary;
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
+- (void) setValue: (id)value forKey: (NSString*)key;
 - (void) takeStoredValue: (id)value forKey: (NSString*)key;
 - (void) takeValue: (id)value forKey: (NSString*)key;
-- (void) setValue: (id)value forKey: (NSString*)key;
 #endif
+/**
+ * Method called by array subscripting.
+ */
+- (void) setObject: (id)anObject forKeyedSubscript: (id)aKey;
+
 @end
 
 #if	defined(__cplusplus)
