@@ -188,14 +188,109 @@ extern "C" {
 @class NSMutableArray;
 @class NSEnumerator;
 @class NSDirectoryEnumerator;
+@class NSError;
+@class NSURL;
+
+/* MacOS-X defines OSType as a 32bit unsigned integer.
+ */
+#ifndef OSTYPE_DECLARED
+typedef	uint32_t	OSType;
+#define OSTYPE_DECLARED
+#endif
 
 @interface NSFileManager : NSObject
 {
+#if	GS_EXPOSE(NSFileManager)
+@private
   NSString	*_lastError;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 
+/**
+ * Returns a shared default file manager which may be used throughout an
+ * application.
+ */
 + (NSFileManager*) defaultManager;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (NSDictionary *) attributesOfItemAtPath: (NSString*)path
+				    error: (NSError**)error;
+
+/**
+ * Copies the item specified by the src path to the 
+ * location specified by the dst path. If the src is a directory,
+ * it is copied recursively with all of its contents.<br />
+ * Errors are returned in the error variable.
+ * Returns YES on success, NO otherwise.
+ */
+- (BOOL) copyItemAtPath: (NSString*)src
+		 toPath: (NSString*)dst
+		  error: (NSError**)error;
+/**
+ * Moves a file or directory specified by src to 
+ * its destination specified by dst, errors are
+ * returned in error.<br />
+ * Returns YES on success, NO otherwise.
+ */
+- (BOOL) moveItemAtPath: (NSString*)src
+		 toPath: (NSString*)dst
+		  error: (NSError**)error;
+
+/**
+ * Removes the file or directory specified by the path
+ * to be removed. If the path points to a directory,
+ * the directory is deleted recursively.<br />
+ * Returns YES on success, otherwise NO.
+ */
+- (BOOL) removeItemAtPath: (NSString*)path
+                    error: (NSError**)error;
+
+/**
+ * Copies the a file or directory specified by the src URL to the 
+ * location specified by the dst URL. If the src is a directory,
+ * it is copied recursively with all of its contents.<br />
+ * Errors are returned in the error variable.
+ * Returns YES on success, NO otherwise.
+ */
+- (BOOL) copyItemAtURL: (NSURL*)src
+		 toURL: (NSURL*)dst
+		 error: (NSError**)error;
+
+/**
+ * Moves a file or directory specified by src to 
+ * its destination specified by dst, errors are
+ * returned in error.<br />
+ * Returns YES on success, NO otherwise.
+ */
+- (BOOL) moveItemAtURL: (NSURL*)src
+		 toURL: (NSURL*)dst
+		 error: (NSError**)error;
+/**
+ * Removes the file or directory specified by the url
+ * to be removed. If the url points to a directory,
+ * the directory is deleted recursively.<br />
+ * Returns YES on success, otherwise NO.
+ */
+- (BOOL) removeItemAtURL: (NSURL*)url
+                   error: (NSError**)error;
+#endif
+
+/**
+ * Changes the current directory used for all subsequent operations.<br />
+ * All non-absolute paths are interpreted relative to this directory.<br />
+ * The current directory is set on a per-task basis, so the current
+ * directory for other file manager instances will also be changed
+ * by this method.
+ */
 - (BOOL) changeCurrentDirectoryPath: (NSString*)path;
 - (BOOL) changeFileAttributes: (NSDictionary*)attributes
 		       atPath: (NSString*)path;
@@ -203,9 +298,29 @@ extern "C" {
 - (NSData*) contentsAtPath: (NSString*)path;
 - (BOOL) contentsEqualAtPath: (NSString*)path1
 		     andPath: (NSString*)path2;
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+/**
+ * Returns an array of the contents of the specified directory.<br />
+ * The listing does <strong>not</strong> recursively list subdirectories.<br />
+ * The special files '.' and '..' are not listed.<br />
+ * Indicates an error by returning nil (eg. if path is not a directory or
+ * it can't be read for some reason).
+ */
+- (NSArray*) contentsOfDirectoryAtPath: (NSString*)path error: (NSError**)error;
+#endif
+
 - (BOOL) copyPath: (NSString*)source
 	   toPath: (NSString*)destination
 	  handler: (id)handler;
+- (BOOL) createDirectoryAtPath: (NSString *)path
+   withIntermediateDirectories: (BOOL)flag
+		    attributes: (NSDictionary *)attributes
+                         error: (NSError **) error;
+- (BOOL) createDirectoryAtURL: (NSURL *)url
+  withIntermediateDirectories: (BOOL)flag
+		   attributes: (NSDictionary *)attributes
+                        error: (NSError **) error;
 - (BOOL) createDirectoryAtPath: (NSString*)path
 		    attributes: (NSDictionary*)attributes;
 - (BOOL) createFileAtPath: (NSString*)path
@@ -216,10 +331,32 @@ extern "C" {
 - (NSString*) currentDirectoryPath;
 - (NSArray*) directoryContentsAtPath: (NSString*)path;
 - (NSString*) displayNameAtPath: (NSString*)path;
+/**
+ * <p>Returns an enumerator which can be used to return each item with
+ * the directory at path in turn.
+ * </p>
+ * <p>The enumeration is recursive ... following all nested subdirectories.
+ * </p>
+ * <p>The order in which directory contents are enumerated is undefined,
+ * and in the current implementation the natural order of the underlying
+ * filesystem is used.
+ * </p>
+ */
 - (NSDirectoryEnumerator*) enumeratorAtPath: (NSString*)path;
 - (NSDictionary*) fileAttributesAtPath: (NSString*)path
 			  traverseLink: (BOOL)flag;
+
+/**
+ * Returns YES if a file (or directory etc) exists at the specified path.
+ */
 - (BOOL) fileExistsAtPath: (NSString*)path;
+
+/**
+ * Returns YES if a file (or directory etc) exists at the specified path.<br />
+ * If the isDirectory argument is not a nul pointer, stores a flag
+ * in the location it points to, indicating whether the file is a
+ * directory or not.<br />
+ */
 - (BOOL) fileExistsAtPath: (NSString*)path isDirectory: (BOOL*)isDirectory;
 - (NSDictionary*) fileSystemAttributesAtPath: (NSString*)path;
 
@@ -247,6 +384,14 @@ extern "C" {
 	   toPath: (NSString*)destination 
 	  handler: (id)handler;
 - (NSString*) pathContentOfSymbolicLinkAtPath: (NSString*)path;
+
+/**
+ * Removes the file or directory at path, using a
+ * handler object which should respond to
+ * [NSObject(NSFileManagerHandler)-fileManager:willProcessPath:] and
+ * [NSObject(NSFileManagerHandler)-fileManager:shouldProceedAfterError:]
+ * messages.
+ */
 - (BOOL) removeFileAtPath: (NSString*)path
 		  handler: (id)handler;
 
@@ -263,7 +408,7 @@ extern "C" {
  * with unicode strings.
  */
 - (NSString*) stringWithFileSystemRepresentation: (const GSNativeChar*)string
-					  length: (unsigned int)len;
+					  length: (NSUInteger)len;
 
 - (NSArray*) subpathsAtPath: (NSString*)path;
 
@@ -326,8 +471,22 @@ extern "C" {
 @end
 
 
+/**
+ *  <p>This is a subclass of <code>NSEnumerator</code> which provides a full
+ *  listing of all the files beneath a directory and its subdirectories.
+ *  Instances can be obtained through [NSFileManager-enumeratorAtPath:].
+ *  </p>
+ *
+ *  <p>This implementation is optimized and performance should be comparable
+ *  to the speed of standard Unix tools for large directories.</p>
+ *
+ *  <p>The order in which directory contents are enumerated is undefined,
+ *  and in the current implementation the natural order of the underlying
+ *  filesystem is used.</p>
+ */
 @interface NSDirectoryEnumerator : NSEnumerator
 {
+#if	GS_EXPOSE(NSDirectoryEnumerator)
 @private
   void *_stack; /* GSIArray */
   NSString *_topPath;
@@ -339,6 +498,16 @@ extern "C" {
     BOOL isFollowing: 1;
     BOOL justContents: 1;
   } _flags;
+#endif
+#if     GS_NONFRAGILE
+#else
+  /* Pointer to private additional data used to avoid breaking ABI
+   * when we don't have the non-fragile ABI available.
+   * Use this mechanism rather than changing the instance variable
+   * layout (see Source/GSInternal.h for details).
+   */
+  @private id _internal GS_UNUSED_IVAR;
+#endif
 }
 - (NSDictionary*) directoryAttributes;
 - (NSDictionary*) fileAttributes;
@@ -449,20 +618,20 @@ GS_EXPORT NSString* const NSFileSystemFreeNodes;
 @interface NSDictionary(NSFileAttributes)
 - (NSDate*) fileCreationDate;
 - (BOOL) fileExtensionHidden;
-- (int) fileHFSCreatorCode;
-- (int) fileHFSTypeCode;
+- (OSType) fileHFSCreatorCode;
+- (OSType) fileHFSTypeCode;
 - (BOOL) fileIsAppendOnly;
 - (BOOL) fileIsImmutable;
 - (unsigned long long) fileSize;
 - (NSString*) fileType;
-- (unsigned long) fileOwnerAccountID;
+- (NSNumber*) fileOwnerAccountID;
 - (NSString*) fileOwnerAccountName;
-- (unsigned long) fileGroupOwnerAccountID;
+- (NSNumber*) fileGroupOwnerAccountID;
 - (NSString*) fileGroupOwnerAccountName;
 - (NSDate*) fileModificationDate;
-- (unsigned long) filePosixPermissions;
-- (unsigned long) fileSystemNumber;
-- (unsigned long) fileSystemFileNumber;
+- (NSUInteger) filePosixPermissions;
+- (NSUInteger) fileSystemNumber;
+- (NSUInteger) fileSystemFileNumber;
 @end
 
 #if	defined(__cplusplus)
