@@ -37,48 +37,56 @@ extern "C" {
 
 #if OS_API_VERSION(GS_API_MACOSX, GS_API_LATEST)
 
-@class NSData, NSString;
+@class NSData, NSString, NSInputStream, NSOutputStream;
 
+enum {
+  NSPropertyListImmutable = 0,
+  NSPropertyListMutableContainers,
+  NSPropertyListMutableContainersAndLeaves
+};
 /**
  * Describes the mutability to use when generating objects during
  * deserialisation of a property list.
+ * <list>
+ * <item><strong>NSPropertyListImmutable</strong>
+ * all objects in created list are immutable</item>
+ * <item><strong>NSPropertyListMutableContainers</strong>
+ * dictionaries, arrays, strings and data objects are mutable</item>
+ * </list>
  */
-typedef enum {
-  NSPropertyListImmutable = 0,
-/** <strong>NSPropertyListImmutable</strong>
- * all objects in created list are immutable
- */
-  NSPropertyListMutableContainers,
-/** <strong>NSPropertyListMutableContainers</strong>
- * dictionaries and arrays are mutable
- */
-  NSPropertyListMutableContainersAndLeaves
-/** <strong>NSPropertyListMutableContainersAndLeaves</strong>
- * dictionaries, arrays, strings and data objects are mutable
- */
-} NSPropertyListMutabilityOptions;
+typedef NSUInteger NSPropertyListMutabilityOptions;
+
+enum {
+  NSPropertyListOpenStepFormat = 1,
+  NSPropertyListXMLFormat_v1_0 = 100,
+  NSPropertyListBinaryFormat_v1_0 = 200,
+
+  NSPropertyListGNUstepFormat = 1000,
+  NSPropertyListGNUstepBinaryFormat
+};
+
+#if OS_API_VERSION(100600,GS_API_LATEST)
+typedef NSUInteger NSPropertyListWriteOptions;
+typedef NSUInteger NSPropertyListReadOptions;
+@class NSError;
+#endif
 
 /**
  * Specifies the serialisation format for a serialised property list.
+ * <list>
+ * <item><strong>NSPropertyListOpenStepFormat</strong>
+ * the most human-readable format</item>
+ * <item><strong>NSPropertyListXMLFormat_v1_0</strong>
+ * portable and readable</item>
+ * <item><strong>NSPropertyListBinaryFormat_v1_0</strong>
+ * the standard format on macos-x</item>
+ * <item><strong>NSPropertyListGNUstepFormat</strong>
+ * extension of OpenStep format</item>
+ * <item><strong>NSPropertyListGNUstepBinaryFormat</strong>
+ * efficient, hardware independent</item>
+ * </list>
  */
-typedef enum {
-  NSPropertyListOpenStepFormat = 1,
-/** <strong>NSPropertyListOpenStepFormat</strong>
- * the most human-readable format */
-  NSPropertyListXMLFormat_v1_0 = 100,
-/** <strong>NSPropertyListXMLFormat_v1_0</strong>
- * portable and readable */
-  NSPropertyListBinaryFormat_v1_0 = 200,
-/** <strong>NSPropertyListBinaryFormat_v1_0</strong>
- * the standard format on macos-x */
-
-  NSPropertyListGNUstepFormat = 1000,
-/** <strong>NSPropertyListGNUstepFormat</strong>
- * extension of OpenStep format */
-  NSPropertyListGNUstepBinaryFormat,
-/** <strong>NSPropertyListGNUstepBinaryFormat</strong>
- * efficient, hardware independent */
-} NSPropertyListFormat;
+typedef NSUInteger NSPropertyListFormat;
 
 /**
  * <p>The NSPropertyListSerialization class provides facilities for
@@ -253,6 +261,26 @@ typedef enum {
 		     format: (NSPropertyListFormat*)aFormat
 	   errorDescription: (NSString**)anErrorString;
 
+#if OS_API_VERSION(100600,GS_API_LATEST)
++ (NSData *) dataWithPropertyList: (id)aPropertyList
+                           format: (NSPropertyListFormat)aFormat
+                          options: (NSPropertyListWriteOptions)anOption
+                            error: (out NSError**)error;
++ (id) propertyListWithData: (NSData*)data
+                    options: (NSPropertyListReadOptions)anOption
+                     format: (NSPropertyListFormat*)aFormat
+                      error: (out NSError**)error;
++ (id) propertyListWithStream: (NSInputStream*)stream
+                      options: (NSPropertyListReadOptions)anOption
+                       format: (NSPropertyListFormat*)aFormat
+                        error: (out NSError**)error;
++ (NSInteger) writePropertyList: (id)aPropertyList
+                       toStream: (NSOutputStream*)stream
+                         format: (NSPropertyListFormat)aFormat
+                        options: (NSPropertyListWriteOptions)anOption
+                          error: (out NSError**)error;
+#endif
+
 @end
 
 #endif	/* GS_API_MACOSX */
