@@ -76,6 +76,15 @@ extern "C" {
  *	GSI_MAP_RELEASE_VAL()
  *		Macro to release the value item in a map table.
  *
+ *	GSI_MAP_WRITE_KEY()
+ *		Macro defining the write barrier for a key.
+ *
+ *	GSI_MAP_WRITE_VAL()
+ *		Macro defining the write barrier for a value.
+ *
+ *	GSI_MAP_READ_KEY()
+ *		Macro defining the read barrier for a key.
+ *
  *	GSI_MAP_HASH()
  *		Macro to get the hash of a key item.
  *
@@ -100,7 +109,7 @@ extern "C" {
  *
  *      GSI_MAP_ZEROED()
  *              Define this macro to check whether a map uses keys which may
- *              be zeroed weak pointers.  This is only used when GC is enabled.
+ *              be zeroed weak pointers.  
  */
 
 #ifndef	GSI_MAP_HAS_VALUE
@@ -132,6 +141,23 @@ extern "C" {
 #ifndef GSI_MAP_ZEROED
 #define GSI_MAP_ZEROED(M)		0
 #endif
+#ifndef GSI_MAP_READ_KEY
+#  define GSI_MAP_READ_KEY(M, x) (*(x))
+#endif
+#ifndef GSI_MAP_READ_VALUE
+#  define GSI_MAP_READ_VALUE(M, x) (*(x))
+#endif
+#ifndef GSI_MAP_WRITE_KEY
+#  define GSI_MAP_WRITE_KEY(M, addr, obj) (*(addr) = obj)
+#endif
+#ifndef GSI_MAP_WRITE_VAL
+#  define GSI_MAP_WRITE_VAL(M, addr, obj) (*(addr) = obj)
+#endif
+#if	GSI_MAP_HAS_VALUE
+#define GSI_MAP_NODE_IS_EMPTY(M, node) (((GSI_MAP_READ_VALUE(M, &node->key).addr) == 0) || ((GSI_MAP_READ_VALUE(M, &node->value).addr == 0)))
+#else
+#define GSI_MAP_NODE_IS_EMPTY(M, node) (((GSI_MAP_READ_VALUE(M, &node->key).addr) == 0))
+#endif
 
 /*
  *      If there is no bitmask defined to supply the types that
@@ -175,9 +201,9 @@ extern "C" {
 
 
 #if (GSI_MAP_KTYPES) & GSUNION_OBJ
-#define GSI_MAP_CLEAR_KEY(node)  node->key.obj = nil
+#define GSI_MAP_CLEAR_KEY(node)  GSI_MAP_WRITE_KEY(map, &node->key, (GSIMapKey)(id)nil)
 #elif  (GSI_MAP_KTYPES) & GSUNION_PTR
-#define GSI_MAP_CLEAR_KEY(node)  node->key.ptr = 0
+#define GSI_MAP_CLEAR_KEY(node)  GSI_MAP_WRITE_KEY(map, &node->key, (GSIMapKey)(void *)NULL)
 #else
 #define GSI_MAP_CLEAR_KEY(node)  
 #endif
@@ -227,9 +253,9 @@ extern "C" {
 #endif
 
 #if (GSI_MAP_VTYPES) & GSUNION_OBJ
-#define GSI_MAP_CLEAR_VAL(node)  node->value.obj = nil
+#define GSI_MAP_CLEAR_VAL(node)  GSI_MAP_WRITE_VAL(map, &node->value, (GSIMapVal)(id)nil)
 #elif  (GSI_MAP_VTYPES) & GSUNION_PTR
-#define GSI_MAP_CLEAR_VAL(node)  node->value.ptr = 0
+#define GSI_MAP_CLEAR_VAL(node)  GSI_MAP_WRITE_VAL(map, &node->value, (GSIMapVal)(void *)NULL)
 #else
 #define GSI_MAP_CLEAR_VAL(node)  
 #endif
@@ -455,12 +481,38 @@ GSIMapRemoveAndFreeNode(GSIMapTable map, uintptr_t bkt, GSIMapNode node)
   return next;
 }
 
+static INLINE void
+GSIMapRemoveWeak(GSIMapTable map)
+{
+  uintptr_t bucketCount = map->bucketCount;
+  GSIMapBucket bucket = map->buckets;
+  if (GSI_MAP_ZEROED(map))
+    {
+      while (bucketCount-- > 0)
+        {
+          GSIMapNode node = bucket->firstNode;
+
+          while (node != 0)
+            {
+              GSIMapNode next = node->nextInBucket;
+              if (GSI_MAP_NODE_IS_EMPTY(map, node))
+                {
+                  GSIMapRemoveNodeFromMap(map, bucket, node);
+                  GSIMapFreeNode(map, node);
+                }
+              node = next;
+            }
+          bucket++;
+        }
+      return;
+    }
+}
+
 static INLINE void
 GSIMapRemangleBuckets(GSIMapTable map,
   GSIMapBucket old_buckets, uintptr_t old_bucketCount,
   GSIMapBucket new_buckets, uintptr_t new_bucketCount)
 {
-#if	GS_WITH_GC
   if (GSI_MAP_ZEROED(map))
     {
       while (old_bucketCount-- > 0)
@@ -469,7 +521,7 @@ GSIMapRemangleBuckets(GSIMapTable map,
 
 	  while ((node = old_buckets->firstNode) != 0)
 	    {
-	      if (node->key.addr == 0)
+	      if (GSI_MAP_NODE_IS_EMPTY(map, node))
 		{
 		  GSIMapRemoveNodeFromMap(map, old_buckets, node);
 		  GSIMapFreeNode(map, node);
@@ -488,7 +540,6 @@ GSIMapRemangleBuckets(GSIMapTable map,
 	}
       return;
     }
-#endif
   while (old_bucketCount-- > 0)
     {
       GSIMapNode	node;
@@ -510,15 +561,15 @@ static INLINE void
 GSIMapMoreNodes(GSIMapTable map, unsigned required)
 {
   GSIMapNode	*newArray;
-  uintptr_t	arraySize = (map->chunkCount+1)*sizeof(GSIMapNode);
 
 #if     GS_WITH_GC
+  uintptr_t	arraySize = (map->chunkCount+1)*sizeof(GSIMapNode);
   /* We don't want our nodes collected before we have finished with them,
    * so we must keep the array of pointers to memory chunks in scanned memory.
    */
   newArray = (GSIMapNode*)NSAllocateCollectable(arraySize, NSScannedOption);
 #else
-  newArray = (GSIMapNode*)NSZoneMalloc(map->zone, arraySize);
+  newArray = (GSIMapNode*)NSZoneCalloc(map->zone, (map->chunkCount+1), sizeof(GSIMapNode));
 #endif
   if (newArray)
     {
@@ -554,7 +605,7 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
       newNodes = GSI_MAP_NODES(map, chunkCount);
 #else
       newNodes
-        = (GSIMapNode)NSZoneMalloc(map->zone, chunkCount*sizeof(GSIMapNode_t));
+        = (GSIMapNode)NSZoneCalloc(map->zone, chunkCount, sizeof(GSIMapNode_t));
 #endif
       if (newNodes)
 	{
@@ -574,14 +625,13 @@ GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 {
   GSIMapNode	node = bucket->firstNode;
 
-#if	GS_WITH_GC
   if (GSI_MAP_ZEROED(map))
     {
-      while ((node != 0) && GSI_MAP_EQUAL(map, node->key, key) == NO)
+      while ((node != 0) && GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
 	{
 	  GSIMapNode	tmp = node->nextInBucket;
 
-	  if (node->key.addr == 0)
+	  if (GSI_MAP_NODE_IS_EMPTY(map, node))
 	    {
 	      GSIMapRemoveNodeFromMap(map, bucket, node);
 	      GSIMapFreeNode(map, node);
@@ -590,8 +640,7 @@ GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 	}
       return node;
     }
-#endif
-  while ((node != 0) && GSI_MAP_EQUAL(map, node->key, key) == NO)
+  while ((node != 0) && GSI_MAP_EQUAL(map, GSI_MAP_READ_KEY(map, &node->key), key) == NO)
     {
       node = node->nextInBucket;
     }
@@ -622,13 +671,12 @@ GSIMapFirstNode(GSIMapTable map)
       uintptr_t		bucket = 0;
       GSIMapNode	node = 0;
 
-#if	GS_WITH_GC
       if (GSI_MAP_ZEROED(map))
 	{
 	  while (bucket < count)
 	    {
 	      node = map->buckets[bucket].firstNode;
-	      while (node != 0 && node->key.addr == 0)
+	      while (node != 0 && GSI_MAP_NODE_IS_EMPTY(map, node))
 		{
 		  node = GSIMapRemoveAndFreeNode(map, bucket, node);
 		}
@@ -640,7 +688,6 @@ GSIMapFirstNode(GSIMapTable map)
 	    }
 	  return node;
 	}
-#endif
       while (bucket < count)
 	{
 	  node = map->buckets[bucket].firstNode;
@@ -676,14 +723,13 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
     }
   bucket = map->buckets + ((unsigned)key.addr) % map->bucketCount;
   node = bucket->firstNode;
-#if	GS_WITH_GC
   if (GSI_MAP_ZEROED(map))
     {
-      while ((node != 0) && node->key.addr != key.addr)
+      while ((node != 0) && GSI_MAP_READ_KEY(map, &node->key).addr != key.addr)
 	{
 	  GSIMapNode	tmp = node->nextInBucket;
 
-	  if (node->key.addr == 0)
+	  if (GSI_MAP_NODE_IS_EMPTY(map, node))
 	    {
 	      GSIMapRemoveNodeFromMap(map, bucket, node);
 	      GSIMapFreeNode(map, node);
@@ -692,8 +738,7 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
 	}
       return node;
     }
-#endif
-  while ((node != 0) && node->key.addr != key.addr)
+  while ((node != 0) && GSI_MAP_READ_KEY(map, &node->key).addr != key.addr)
     {
       node = node->nextInBucket;
     }
@@ -802,14 +847,13 @@ GSIMapEnumeratorForMap(GSIMapTable map)
   /*
    * Locate next bucket and node to be returned.
    */
-#if	GS_WITH_GC
   if (GSI_MAP_ZEROED(map))
     {
       while (enumerator.bucket < map->bucketCount)
 	{
 	  GSIMapNode	node = map->buckets[enumerator.bucket].firstNode;
 
-	  while (node != 0 && node->key.addr == 0)
+	  while (node != 0 && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
 	    {
 	      node = GSIMapRemoveAndFreeNode(map, enumerator.bucket, node);
 	    }
@@ -820,7 +864,6 @@ GSIMapEnumeratorForMap(GSIMapTable map)
 	  enumerator.bucket++;
 	}
     }
-#endif
   while (enumerator.bucket < map->bucketCount)
     {
       enumerator.node = map->buckets[enumerator.bucket].firstNode;
@@ -872,21 +915,20 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
   GSIMapNode	node = ((_GSIE)enumerator)->node;
   GSIMapTable	map = ((_GSIE)enumerator)->map;
 
-#if	GS_WITH_GC
   /* Find the frst available non-zeroed node.
    */
-  if (node != 0 && GSI_MAP_ZEROED(map) && node->key.addr == 0)
+  if (node != 0 && GSI_MAP_ZEROED(map) && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
     {
       uintptr_t		bucketCount = map->bucketCount;
       uintptr_t		bucket = ((_GSIE)enumerator)->bucket;
 
-      while (node != 0 && node->key.addr == 0)
+      while (node != 0 && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
 	{
 	  node = GSIMapRemoveAndFreeNode(map, bucket, node);
 	  while (node == 0 && ++bucket < bucketCount)
 	    {
 	      node = (map->buckets[bucket]).firstNode;
-	      while (node != 0 && node->key.addr == 0)
+	      while (node != 0 && GSI_MAP_READ_KEY(map, &node->key).addr == 0)
 		{
 		  node = GSIMapRemoveAndFreeNode(map, bucket, node);
 		}
@@ -895,13 +937,11 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 	  ((_GSIE)enumerator)->node = node;
 	}
     }
-#endif
 
   if (node != 0)
     {
       GSIMapNode	next = node->nextInBucket;
 
-#if	GS_WITH_GC
       if (GSI_MAP_ZEROED(map))
 	{
 	  uintptr_t	bucket = ((_GSIE)enumerator)->bucket;
@@ -911,14 +951,12 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 	      next = GSIMapRemoveAndFreeNode(map, bucket, next);
 	    }
 	}
-#endif
 
       if (next == 0)
 	{
 	  uintptr_t	bucketCount = map->bucketCount;
 	  uintptr_t	bucket = ((_GSIE)enumerator)->bucket;
 
-#if	GS_WITH_GC
 	  if (GSI_MAP_ZEROED(map))
 	    {
 	      while (next == 0 && ++bucket < bucketCount)
@@ -933,7 +971,6 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 	      ((_GSIE)enumerator)->node = next;
 	      return node;
 	    }
-#endif
 	  while (next == 0 && ++bucket < bucketCount)
 	    {
 	      next = (map->buckets[bucket]).firstNode;
@@ -993,7 +1030,7 @@ GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
            * will only work with things that are id-sized, however, so don't
            * try using it with non-object collections.
            */
-          stackbuf[i] = *(id*)&node->key.bl;
+          stackbuf[i] = (id)GSI_MAP_READ_KEY(map, &node->key).addr;
         }
     }
   /* Store the important bits of the enumerator in the caller. */
@@ -1021,8 +1058,8 @@ GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 	}
     }
   map->freeNodes = node->nextInBucket;
-  node->key = key;
-  node->value = value;
+  GSI_MAP_WRITE_KEY(map, &node->key, key);
+  GSI_MAP_WRITE_VAL(map, &node->value, value);
   node->nextInBucket = 0;
   GSIMapRightSizeMap(map, map->nodeCount);
   GSIMapAddNodeToMap(map, node);
@@ -1044,9 +1081,9 @@ GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 	}
     }
   map->freeNodes = node->nextInBucket;
-  node->key = key;
-  node->value = value;
+  GSI_MAP_WRITE_KEY(map, &node->key, key);
   GSI_MAP_RETAIN_KEY(map, node->key);
+  GSI_MAP_WRITE_VAL(map, &node->value, value);
   GSI_MAP_RETAIN_VAL(map, node->value);
   node->nextInBucket = 0;
   GSIMapRightSizeMap(map, map->nodeCount);
@@ -1069,7 +1106,7 @@ GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
 	}
     }
   map->freeNodes = node->nextInBucket;
-  node->key = key;
+  GSI_MAP_WRITE_KEY(map, &node->key, key);
   node->nextInBucket = 0;
   GSIMapRightSizeMap(map, map->nodeCount);
   GSIMapAddNodeToMap(map, node);
@@ -1091,7 +1128,7 @@ GSIMapAddKey(GSIMapTable map, GSIMapKey key)
 	}
     }
   map->freeNodes = node->nextInBucket;
-  node->key = key;
+  GSI_MAP_WRITE_KEY(map, &node->key, key);
   GSI_MAP_RETAIN_KEY(map, node->key);
   node->nextInBucket = 0;
   GSIMapRightSizeMap(map, map->nodeCount);
@@ -1146,9 +1183,11 @@ GSIMapCleanMap(GSIMapTable map)
 	  while(node != 0)
 	    {
 	      GSI_MAP_RELEASE_KEY(map, node->key);
+	      GSI_MAP_CLEAR_KEY(node);
 	  
 #if	GSI_MAP_HAS_VALUE
 	      GSI_MAP_RELEASE_VAL(map, node->value);
+	      GSI_MAP_CLEAR_VAL(node);
 #endif
 	      prevNode = node;
 	      node = node->nextInBucket;
