@@ -3,24 +3,24 @@
 
    Written by: Richard Frith-Macdonald <rfm@gnu.org>
    Date: Oct, October 2006
-   
+
    This file is part of GNUstep.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
-*/ 
+*/
 
 #ifndef __GNUSTEP_GSVERSIONMACROS_H_INCLUDED_
 #define __GNUSTEP_GSVERSIONMACROS_H_INCLUDED_
@@ -91,7 +91,7 @@
  * two digits are the major version number, the second two are the minor
  * version number and the last two are the subminor number (all left padded
  * with a zero where necessary).  However, for convenience you can also
- * use any of several predefined constants ... 
+ * use any of several predefined constants ...
  * <ref type="macro" id="GS_API_NONE">GS_API_NONE</ref>,
  * <ref type="macro" id="GS_API_LATEST">GS_API_LATEST</ref>,
  * <ref type="macro" id="GS_API_OSSPEC">GS_API_OSSPEC</ref>,
@@ -124,7 +124,7 @@
  * two digits are the major version number, the second two are the minor
  * version number and the last two are the subminor number (all left padded
  * with a zero where necessary).  However, for convenience you can also
- * use any of several predefined constants ... 
+ * use any of several predefined constants ...
  * <ref type="macro" id="GS_API_NONE">GS_API_NONE</ref>,
  * <ref type="macro" id="GS_API_LATEST">GS_API_LATEST</ref>,
  * <ref type="macro" id="GS_API_OSSPEC">GS_API_OSSPEC</ref>,
@@ -193,6 +193,35 @@
 #import <GNUstepBase/GSConfig.h>
 #endif
 
+
+/* Attribute definitions for attributes which may or may not be supported
+ * depending on the compiler being used.
+ * NB we currently expect gcc to be version 4 or later.
+ *
+ * The definition should be of the form GS_XXX_CONTEXT where XXX is the
+ * name of the attribute and CONTEXT is one of FUNC, METH, or IVAR
+ * depending on where the attribute can be applied.
+ */
+
+#if __GNUC__*10+__GNUC_MINOR__ >= 31
+#  define GS_DEPRECATED_FUNC __attribute__ ((deprecated))
+#else
+#  define GS_DEPRECATED_FUNC
+#endif
+
+#define GS_UNUSED_ARG __attribute__((unused))
+
+#define GS_UNUSED_FUNC __attribute__((unused))
+
+// FIXME ... what version of gcc?
+#if __clang__
+#  define GS_UNUSED_IVAR __attribute__((unused))
+#else
+#  define GS_UNUSED_IVAR 
+#endif
+
+
+
 #ifndef __has_feature
 #define __has_feature(x) 0
 #endif
@@ -228,11 +257,87 @@
 
 #define	GS_EXPOSE(X)	(!GS_NONFRAGILE || defined(EXPOSE_##X##_IVARS))
 
+/* Static analyser macros: Provide annotations to help the analyser */
+#ifdef __clang__
+#  define GS_NORETURN_METHOD __attribute__((__noreturn__))
+#else
+#  define GS_NORETURN_METHOD
+#endif
+
+#ifndef NS_RETURNS_RETAINED
+#  if __has_feature(attribute_ns_returns_retained)
+#    define NS_RETURNS_RETAINED __attribute__((ns_returns_retained))
+#  else
+#    define NS_RETURNS_RETAINED
+#  endif
+#endif
+
+#ifndef NS_RETURNS_NOT_RETAINED
+#  if __has_feature(attribute_ns_returns_not_retained)
+#    define NS_RETURNS_NOT_RETAINED __attribute__((ns_returns_not_retained))
+#  else
+#    define NS_RETURNS_NOT_RETAINED
+#  endif
+#endif
+
+#ifndef NS_CONSUMED
+#  if __has_feature(attribute_ns_consumed)
+#    define NS_CONSUMED __attribute__((ns_consumed))
+#  else
+#    define NS_CONSUMED
+#  endif
+#endif
+
+
+#ifndef NS_CONSUMES_SELF
+#  if __has_feature(attribute_ns_consumes_self)
+#    define NS_CONSUMES_SELF __attribute__((ns_consumes_self))
+#  else
+#    define NS_CONSUMES_SELF
+#  endif
+#endif
+
+#if defined(__clang__) && defined(__OBJC__)
+static inline void gs_consumed(id NS_CONSUMED o) GS_UNUSED_FUNC;
+static inline void gs_consumed(id NS_CONSUMED GS_UNUSED_ARG o) { return; }
+#define	GS_CONSUMED(O)	gs_consumed(O);
+#else
+#define	GS_CONSUMED(O)
+#endif
+
+/* Include the appropriate header for ObjC2 blocks support if it is in use.
+ *
+ * FIXME: "OBJC2RUNTIME" is set to "1" if the runtime supports the ObjC2
+ * runtime API, which is unrelated to whether the compiler has blocks
+ * support or not.
+ */
+#if __has_feature(blocks)
+#  if	OBJC2RUNTIME
+#    if defined(_APPLE_)
+#      include <Block.h>
+#    else
+#      include <objc/blocks_runtime.h>
+#    endif
+#  else
+#    include <ObjectiveC2/blocks_runtime.h>
+#  endif
+#endif
+
+/* Attribute definition for root classes, annotates the interface declaration
+ * of the class.
+ */
+#ifndef GS_ROOT_CLASS
+#  if GS_HAVE_OBJC_ROOT_CLASS_ATTR || __has_feature(attribute_objc_root_class)
+#    define GS_ROOT_CLASS __attribute__((objc_root_class))
+#  else
+#    define GS_ROOT_CLASS
+#  endif
+#endif
 
 
 
 #if	defined(GNUSTEP_WITH_DLL)
- 
+
 #if BUILD_libgnustep_base_DLL
 #
 # if defined(__MINGW__)
@@ -240,7 +345,7 @@
    * __declspec(dllexport) is not needed.
    */
 #  define GS_EXPORT  extern
-#  define GS_DECLARE 
+#  define GS_DECLARE
 # else
 #  define GS_EXPORT  __declspec(dllexport)
 #  define GS_DECLARE __declspec(dllexport)
@@ -249,19 +354,31 @@
 #  define GS_EXPORT  extern __declspec(dllimport)
 #  define GS_DECLARE __declspec(dllimport)
 #endif
- 
+
 #else /* GNUSTEP_WITH[OUT]_DLL */
 
 #  define GS_EXPORT extern
-#  define GS_DECLARE 
+#  define GS_DECLARE
 
 #endif
 
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1))
-#define GS_ATTRIB_DEPRECATED __attribute__ ((deprecated))
+
+/* Attribute macros compatible with Apple.
+ */
+
+#if __GNUC__*10+__GNUC_MINOR__ >= 42
+#  define NS_FORMAT_ARGUMENT(A) __attribute__((format_arg(A)))
 #else
-#define GS_ATTRIB_DEPRECATED
+#  define NS_FORMAT_ARGUMENT(F,A) 
 #endif
 
+#if __GNUC__*10+__GNUC_MINOR__ >= 42
+#  define NS_FORMAT_FUNCTION(F,A) __attribute__((format(__NSString__, F, A)))
+#else
+#  define NS_FORMAT_FUNCTION(F,A) 
+#endif
+
+#define NS_REQUIRES_NIL_TERMINATION __attribute__((sentinel))
+
 
 #endif /* __GNUSTEP_GSVERSIONMACROS_H_INCLUDED_ */
